#!/usr/bin/env python

# You should probably run this as a cron job every hour or so

from __future__ import generators

import blog_config, db_row, psycopg, urlutils, sql, blog, urllib2, weblogs
import urlparse, re, robotparser
from optparse import OptionParser
from mx import DateTime

UA = '%s (blogroll update notification module; %s)' % (blog.LS_BLOG_VERSION,
                                                       blog.LS_BLOG_URL)
def manual_update_check(url, updated=None):
    if not urlutils.robot_can_fetch(url, ua=UA):
        return None
    
    req = urllib2.Request(url)
    if updated:
        req.add_header('If-Modified-Since', DateTime.ARPA.strGMT(updated))
    req.add_header('User-Agent', UA)
    
    fp = urlutils.open_url(req)
    if not fp:
        return None

    lmt = fp.headers.get('last-modified', '')
    fp.close()
    if lmt:
        return ('', DateTime.ARPA.ParseDateTimeUTC(lmt))
    return None

_indexre = re.compile(r'index.(html|htm|shtml|php|pl|cgi)$', re.I)
def possible_urls(baseurl, nodeep=False):
    "Cheesy implementation of trying to deal with all the possible "
    "permutations of a particular URL"
    yield baseurl

    if baseurl[-1] == '/':
        yield baseurl[:-1]
        yield baseurl+'index.html'
        yield baseurl+'index.htm'
        yield baseurl+'index.php'
    else:
        yield baseurl+'/'

    m = _indexre.match(baseurl)
    if m:
        yield _indexre.sub('', baseurl)

    if nodeep:
        return

    bits = list(urlparse.urlsplit(baseurl))
    if bits[1].startswith('www.'):
        bits[1] = bits[1][4:]
    else:
        bits[1] = 'www.'+bits[1]

    baseurl = urlparse.urlunsplit(bits)
    for url in possible_urls(baseurl, nodeep=True):
        yield url

def main():
    usage = 'usage: %prog [options] url1 url2...'
    parser = OptionParser(usage=usage,
                          version='%prog '+blog.LS_BLOG_VERSION)
    
    parser.add_option('-d', '-v', '--debug', dest='debug', action='store_true',
                      default=False, help='display debugging output')
    parser.add_option('-m', '--manual-check', dest='manual',
                      action='store_true', default=False,
                      help='check sites on blogroll for last modified time')
    (options, args) = parser.parse_args()

    DEBUG = options.debug
    MANUAL_CHECK = options.manual

    if args:
        URLS = args
    else:
        URLS = [weblogs.URL, 'http://blo.gs/changes60.xml']

    db = psycopg.connect(blog_config.DSN)
    db.commit()
    cursor=db.cursor()
    cursor.execute('SELECT MAX(updated) FROM blogroll')
    lastmod = cursor.fetchone()[0]
    ims = DateTime.ARPA.strGMT(lastmod)
    if DEBUG:
        print 'If-Modified-Since:', ims

    blogs = {}
    for url in URLS:
        # Should return an empty dictionary if not modified
        if DEBUG:
            print 'Fetching updates from', url
        blogs.update(weblogs.get_weblogs_updates(url, UA, ims))
    
    if blogs:
        cursor.execute("SELECT oid, name, url, "
                       "updated AT TIME ZONE 'UTC' AS updated FROM blogroll")
        R = db_row.MetaRow(cursor.description)
        blogroll = [R(row) for row in cursor.fetchall()]

        for entry in blogroll:
            bits = list(urlparse.urlsplit(entry.fields.url))
            bits[1] = bits[1].lower()
            baseurl = urlparse.urlunsplit(bits)
            for url in possible_urls(baseurl):
                info = blogs.get(url)
                if info:
                    if DEBUG and url != entry.fields.url:
                        print url
                    break
            if not info and MANUAL_CHECK:
                info = manual_update_check(entry.fields.url, entry.fields.updated)
            if not info:
                continue

            name, updated = info 
            if DEBUG:
                print `entry.fields.name`, updated, `entry.fields.name == name`
            if updated and updated > entry.fields.updated:
                query = sql.Update('blogroll',
                                   {'updated' : DateTime.ISO.strUTC(updated)},
                                   {'oid' : entry.fields.oid})
                if DEBUG:
                    print query
                cursor.execute(str(query))
                db.commit()

if __name__ == '__main__':
    main()
