#!/usr/bin/python2.2
import psycopg
import blog_config

db = psycopg.connect(blog_config.DSN)
db.autocommit(1)
cursor = db.cursor()

# Authors table
cursor.execute('CREATE TABLE authors (author SERIAL PRIMARY KEY,'
               'authorname VARCHAR(60) NOT NULL, authoremail VARCHAR(80),'
               'authorurl VARCHAR(80), login VARCHAR(16), '
               # Airport codes
               'icao VARCHAR(4), iata VARCHAR(3), '
               # Does the author have admin privs (currently useless)
               'admin BOOLEAN DEFAULT FALSE)')

# Topics table
cursor.execute('CREATE TABLE topics (topic SERIAL PRIMARY KEY,'
               'topicname VARCHAR(80) NOT NULL, topicdesc TEXT,'
               'odtopic VARCHAR(1024))')

# Content table
cursor.execute('CREATE TABLE content (entryid SERIAL PRIMARY KEY,'
               'title VARCHAR(100) NOT NULL,'
               'body TEXT NOT NULL, extended TEXT, excerpt TEXT,'
               'author INTEGER REFERENCES authors,'
               'date TIMESTAMP (0) WITH TIME ZONE DEFAULT now(),'
               'lastmod TIMESTAMP (0) WITH TIME ZONE DEFAULT now(),'
               'visible BOOLEAN NOT NULL DEFAULT TRUE,'
               'pinned BOOLEAN NOT NULL DEFAULT FALSE)')

# Topic-entry mapping
cursor.execute('CREATE TABLE topicmap (entryid INTEGER REFERENCES content,'
               'topic INTEGER REFERENCES topics) UNIQUE (entryid, topics)')

# Trackback table
cursor.execute('CREATE TABLE trackback (entryid INTEGER REFERENCES content,'
               'title VARCHAR(128), url VARCHAR(128), excerpt VARCHAR(256),'
               'blog_name VARCHAR(128), added TIMESTAMP (0) WITH TIME ZONE '
               'DEFAULT now(), ip INET)')

# Blogroll table
cursor.execute("CREATE TABLE blogroll (name VARCHAR(128) NOT NULL, "
               "url VARCHAR(128) NOT NULL, comment VARCHAR(128) DEFAULT '', "
               "rssurl VARCHAR(160), "
               "updated TIMESTAMP (0) WITH TIME ZONE DEFAULT '2001-01-01')")

# Link cosmos table
cursor.execute('CREATE TABLE cosmos (name VARCHAR(128), url VARCHAR(128), '
               'rssurl VARCHAR(128), inboundblogs INTEGER, '
               'inboundlinks INTEGER)')

# Outgoing trackbacks table
cursor.execute('CREATE TABLE pinged (entryid INTEGER REFERENCES content, '
               'tburl VARCHAR(128))')

# Best of the blog table
cursor.execute('CREATE TABLE best (entryid INTEGER UNIQUE REFERENCES content)')

cursor.execute('CREATE INDEX trackback_index ON trackback (entryid)')
cursor.execute('CREATE INDEX topicmap_index ON topicmap (entryid)')
cursor.execute('CREATE INDEX topic_index ON topicmap (topic)')
cursor.execute('CREATE INDEX content_author_index ON content (author)')
cursor.execute('CREATE INDEX content_date_index ON content (date)')

cursor.execute('GRANT SELECT ON trackback, blogroll, authors, topics, content, cosmos, topicmap, pinged, best TO PUBLIC')
cursor.execute('GRANT ALL ON trackback, blogroll, authors, topics, content, cosmos, topicmap, pinged, best TO "www-data"')
db.close()
