#!/usr/bin/python

import urllib2, urllib, HTMLParser, re, urlparse, blog, urlutils
import syslog, sys, robotparser
import time
from xml.dom.minidom import parse, parseString
from blog_config import *

LEGACY_MODE = True
DEBUG = True

OBEY_ROBOTS_TXT = False

UA = '%s (trackback module; %s)' % (blog.LS_BLOG_VERSION, blog.LS_BLOG_URL)

# Usage:
# send_ping("http://whereever.com/",
#           title="Lame entry", blog_name="lameness"...)
def send_ping(tb_url, rp=None, **query):
    # LEGACY_MODE uses GETs; the new (1.1) mode uses POSTs.
    if LEGACY_MODE and '?' in tb_url:
        data = None
        tb_url += '&'+urllib.urlencode(query)
    else:
        data = urllib.urlencode(query)

    if rp:
        if not rp.can_fetch('Python-urllib/%s, %s' % (urllib2.__version__, UA),
                            tb_url):
            return False
        
    req = urllib2.Request(tb_url, data, headers={'User-Agent' : UA,})
    if 'url' in query:
        req.add_header('Referer', query['url'])

    fob = urlutils.open_url(req)
    #print >> sys.stderr, repr(data), tb_url, repr(query), fob
    if not fob:
        return
    response = fob.read()
    fob.close()
    if DEBUG:
        fp = open('/tmp/tb.log', 'a')
        print >> fp, 'Trackback URL:', tb_url, time.ctime()
        if data:
            print >> fp, repr(data)
        print >> fp, response
        fp.close()

    try:
        dom = parseString(response)
        response = dom.getElementsByTagName("response")[0]
        error = response.getElementsByTagName("error")[0]
    except:
        pass
    #if error.data == '1':
    #    return response.getElementsByTagName("message")[0].data
    #else:
    #    return None
    return True

# I probably should replace this with a real xml parser
rdfre = re.compile(r'<rdf:RDF(.*?)</rdf:RDF>', re.MULTILINE | re.DOTALL)
identre = re.compile(r'dc:identifier="([^"]+)"')
tbpingre = re.compile(r'trackback:ping="([^"]+)"')
titlere = re.compile(r'dc:title="([^"]+)"')
aboutre = re.compile(r'about="([^"]+)"')
fragre = re.compile(r'#.*$')

def urls_match(url1, url2):
    """Return True if the two URLs are canonically the same."""
    if url1 == url2:
        return True

    x1 = list(urlparse.urlsplit(url1))
    x2 = list(urlparse.urlsplit(url2))

    x1[1] = x1[1].lower()
    x2[1] = x2[1].lower()
    return x1 == x2

def discover_tb(url, referrer=None, rp=None, returnall=False):
    if rp:
        if not rp.can_fetch('Python-urllib/%s, %s' % (urllib2.__version__, UA),
                            url):
            return None

    if returnall:
        entries = []

    no_frag_url = fragre.sub('', url)
    req = urllib2.Request(url, headers={'User-Agent' : UA})
    # Send our URL too, so we show up in their referrer log even if they
    # don't have a trackback URL embedded in the page...
    #
    # NB: I don't consider this referrer abuse, since it sends the entry
    # URL, not an "ad" URL for the product.
    if referrer:
        req.add_header('Referer', referrer)

    response = urlutils.open_url(req)
    
    if not response:
        return None
    content = response.read()
    for match in rdfre.finditer(content):
        if not match: continue
        
        rdf = match.group(1)
        match = identre.search(rdf)
        if not match: continue
        
        perm_url = urlparse.urljoin(url, match.group(1))
        if returnall:
            match = titlere.search(rdf)
            if not match:
                continue
            title = match.group(1)
            match = tbpingre.search(rdf)
            if not match:
                continue
            #match = aboutre.search(rdf)
            #if match:
            entries.append((perm_url, title))
            continue
        elif urls_match(perm_url, url) or urls_match(perm_url, no_frag_url):
            match = tbpingre.search(rdf)
            if match: return match.group(1)
            match = aboutre.search(rdf)
            if match: return match.group(1)

    if returnall:
        return entries
    return None

class HrefParser(HTMLParser.HTMLParser):
    def reset(self):
        HTMLParser.HTMLParser.reset(self)
        self.urls = []

    def handle_starttag(self, tag, attrs):
        if tag == 'a':
            for item in attrs:
                attr, val = item
                if attr == 'href':
                    self.urls.append(val)

def grab_embedded_urls(content):
    p = HrefParser()
    p.feed(content)
    p.close()
    return p.urls

def ping_embedded_urls(content, pinged=None, **data):
    if not pinged:
        pinged = {}
    elif not isinstance(pinged, dict):
        pinged = dict([(x, True) for x in pinged])

    for url in grab_embedded_urls(content):
        # Fix up any relative URLs
        url = urlparse.urljoin(URL, url)
        if OBEY_ROBOTS_TXT:
            bits = list(urlparse.urlsplit(url))
            bits[2:] = ['robots.txt', '', '']
            robots_url = urlparse.urlunsplit(tuple(bits))
            rp = robotparser.RobotFileParser()
            rp.set_url(robots_url)
            try:
                rp.read()
            except (IOError, urlutils.Timeout):
                rp = None
        else:
            rp = None
        tb_url = discover_tb(url, data['url'], rp)
        #print >> sys.stderr, url, tb_url
        if tb_url:
            # Only ping a particular trackback URL once...
            if tb_url in pinged:
                continue
            
            pinged[tb_url] = True
            send_ping(tb_url, rp, **data)
        del rp

    return pinged.keys()

def prompt_ping():
    url = raw_input('Site to ping: ')
    if not url: return
    myurl = raw_input('Local URL: ')
    if not myurl: return
    tburl = discover_tb(url, myurl)
    if not tburl: return
    print 'Trackback:', tburl
    title = raw_input('Title: ')
    if not title: return
    excerpt = raw_input('Excerpt: ')
    if not excerpt: return

    send_ping(tburl, url=myurl, title=title, excerpt=excerpt,
              blog_name=BLOG_TITLE)
    return

if __name__ == '__main__':
    prompt_ping()
    #print discover_tb('http://www.scrappleface.com/MT/archives/000608.html#000608',
    #                  'http://www.i69info.com/')
    #print discover_tb('http://www.cleverhack.com/blog/archives/001124.html',
    #                  'http://blog.lordsutch.com/')
    #url = discover_tb('http://blog.lordsutch.com/?entryid=1')
    #send_ping(url, url="http://blog.lordsutch.com/?entryid=2", title="Test",
    #               blog_name="Chris's blog", excerpt="Blah blah.")
