#!/usr/bin/env python

# Code to ping weblogs.com and other sites with compatible interfaces
# Also includes code to query weblogs.com's changes file

## (C) 2003 Chris Lawrence <chris@lordsutch.com>.  All Rights Reserved.
##
## Permission to use, copy, modify, and distribute this software and
## its documentation for any purpose and without fee is hereby
## granted, provided that the above copyright notice appear in all
## copies and that both that copyright notice and this permission
## notice appear in supporting documentation, and that the name of
## Chris Lawrence not be used in advertising or publicity pertaining
## to distribution of the software without specific, written prior
## permission.
##
## CHRIS LAWRENCE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
## SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
## FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
## FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
## WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
## AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
## OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
## SOFTWARE.

import sys, xml.sax, urllib2, urlutils, urlparse, xmlrpclib
from mx import DateTime

SERVER = 'http://rpc.weblogs.com/RPC2'
URL = "http://www.weblogs.com/changes.xml"
UA = 'weblogs.py'

class WeblogUpdatesHandler(xml.sax.handler.ContentHandler):
    def __init__(self):
        self.updated = DateTime.gmtime()
        self.blogs = {}

    def startElement(self, name, attrs):
        if name == 'weblogUpdates':
            updated = attrs.get('updated')
            #if updated:
            #    self.updated = DateTime.ARPA.ParseDateTimeUTC(updated)
        elif name == 'weblog':
            url = attrs.get('url')
            if url:
                name = attrs.get('name', '')
                when = int(attrs.get('when', 0))
                # Normalize the host part of the URL to lowercase
                bits = list(urlparse.urlsplit(url))
                bits[1] = bits[1].lower()
                url = urlparse.urlunsplit(bits)
                self.blogs[url] = (name, self.updated-DateTime.DateTimeDeltaFromSeconds(when))

def get_weblogs_updates(req=URL, ua=UA, if_mod_since=None):
    "Returns a dictionary {'url' : ('name', DateTime(last_updated))}"
    if not isinstance(req, urllib2.Request):
        req = urllib2.Request(req)

    req.add_header('User-Agent', UA)
    if if_mod_since:
        req.add_header('If-Modified-Since', if_mod_since)

    try:
        fob = urlutils.open_url(req)
    except urllib2.HTTPError, x:
        if x.code == 304:
            return {}
        raise
    if not fob:
        return {}
    data = fob.read()
    fob.close()

    # Fix broken blo.gs feed
    data = data.replace('\x01', '')
    
    handler = WeblogUpdatesHandler()
    try:
        xml.sax.parseString(data, handler)
    except xml.sax.SAXParseException:
        pass
    fob.close()
    
    return handler.blogs

def ping(weblogName, weblogURL, serverURL=SERVER, checkURL='', rssURL=''):
    server = xmlrpclib.ServerProxy(serverURL)
    result = False
    if checkURL or rssURL:
        try:
            response = server.weblogUpdates.extendedPing(weblogName, weblogURL,
                                                         checkURL, rssURL)
            result = not result['flerror']
        except:
            result = False

    if not result:
        try:
            response = server.weblogUpdates.ping(weblogName, weblogURL)
            result = not result['flerror']
        except:
            result = False
    return result

URLS = [SERVER, 'http://ping.blo.gs/', 'http://rpc.blogrolling.com/pinger/',
        'http://ping.rootblog.com/rpc.php']

def ping_all(weblogName, weblogURL):
    for site in URLS:
        ping(weblogName, weblogURL, site)

if __name__ == "__main__":
    print ping(sys.argv[1], sys.argv[2])
