#!/usr/bin/python
# -*- python -*-
#
# A thoroughly lame weblog: CGI Interface
# (C) 2002-03 Chris Lawrence

# Theory of operation:
# - Use postgresql to do the heavy lifting; that's why we have databases
#
# Tables:
# - content
# - topics
# - authors

import cgitb; cgitb.enable()

import sys, cgi, psycopg, re, timing, urllib, os, Cookie, cStringIO
import blog

from blog_config import *
from blog import *
from mx import DateTime
from ezFieldStorage import ezFieldStorage

def main():
    #path_query=os.environ.get('PATH_INFO')
    referrer = os.environ.get('HTTP_REFERER', '')
    localref = referrer and referrer.startswith(URL)
    
    cookie_str = os.environ.get('HTTP_COOKIE', '')
    try:
        cookies = Cookie.BaseCookie(cookie_str)
    except CookieError:
        cookies = Cookie.BaseCookie()

    try:
        db = psycopg.connect(DSN)
    except:
        failed_query('Database is currently down; please try again in a few '
                     'minutes.')
        return

    queries = []
    form = cgi.FieldStorage()
    topic = form.getfirst('topic')
    if topic:
        try:
            topic = int(topic)
            queries.append( "c.topic=%d" % topic )
        except:
            failed_query('Invalid topic ID.')
            return

    author = form.getfirst('author')
    if author:
        try:
            author = int(author)
            queries.append("c.author=%d" % author)
        except:
            failed_query('Invalid author.')
            return

    limit = form.getfirst('limit')
    if limit:
        try:
            limit = int(limit)
        except:
            pass
    limit = limit or DEFLIMIT
    # Make sure limit is in [1, 50]
    limit = min(max(1, limit), 50)

    offset = form.getfirst('offset')
    if offset:
        try:
            offset = int(offset)
        except:
            pass
    # Make sure the offset >=0
    offset = max(offset or 0, 0)

    entryid = form.getfirst('entryid')
    if entryid:
        try:
            queries.append('c.entryid=%d' % int(entryid))
        except:
            pass

    start = form.getfirst('start')
    if start:
        try:
            queries.append('c.entryid <= %d' % int(start))
        except:
            pass

    month = form.getfirst('month')
    if month:
        try:
            month = int(month)
            y, m = divmod(month, 100)
            queries.extend(['extract(YEAR FROM c.date)=%d' % y,
                            'extract(MONTH FROM c.date)=%d' % m])
        except:
            pass

    frontpage = not len(queries)
        
    # overview=1 shows the latest post on each topic
    distinct = 'ALL'
    distorder = 'c.date DESC'
    overview = form.getfirst('overview')
    if overview:
        try:
            overview = int(overview)
            if overview:
                distinct = 'DISTINCT ON (c.topic)'
                distorder = 'c.topic, date DESC'
                frontpage = False
        except:
            pass

    basequeries = ['c.topic=t.topic', 'c.author=a.author', 'c.visible']

    queries.extend(basequeries)
    query = ' AND '.join(queries)

    cursor = db.cursor()
    cursor.execute("SET TIME ZONE 'UTC'")
    cursor.execute("SELECT %s c.lastmod FROM content c, authors a, topics t WHERE %s ORDER BY %s" % (distinct, query, distorder))
    count = cursor.rowcount

    if not count:
        failed_query('No entries matching criteria.')
        return

    latest = DateTime.DateTimeFromJDN(0)
    entries = cursor.fetchall()
    for entry in entries:
        date = entry[0]
        latest = max(date, latest)
    expires = DateTime.gmt()+(DateTime.oneMinute*TTL)
    extras = [ ('Last-Modified', DateTime.ARPA.strGMT(latest)),
               ('Expires', DateTime.ARPA.strGMT(expires)) ]

    cursor.execute('SELECT topic, topicname, topicdesc FROM topics ORDER BY topicname')
    topics = cursor.fetchall()
    
    if TIMEZONE:
        cursor.execute("SET TIME ZONE '%s'" % TIMEZONE)

    cursor.execute("SELECT %s c.entryid, c.title, c.body, a.authorname,"
                   "c.author, c.topic, t.topicname, t.topicdesc,"
                   "to_char(c.date, %s),a.authoremail, to_char(c.lastmod, %s),"
                   "c.date, c.lastmod FROM content c, topics t, authors a "
                   "WHERE %s ORDER BY %s LIMIT %d OFFSET %d" % (
        distinct, DATEFMT, DATEFMT, query, distorder, limit, offset))
    entries = cursor.fetchall()

    query = ' AND '.join(basequeries)
    cursor.execute(
        "SELECT c.entryid, c.title, c.body, a.authorname,"
        "c.author, c.topic, t.topicname, t.topicdesc,"
        "to_char(c.date, %s), a.authoremail, to_char(c.lastmod, %s), c.date, "
        "c.lastmod FROM content c, topics t, authors a "
        "WHERE %s ORDER BY c.lastmod DESC LIMIT %d" % (
        DATEFMT, DATEFMT, query, UPDATED) )
    recent = cursor.fetchall()

    tbcount = {}
    for entry in entries:
        eid = entry[0]
        cursor.execute('SELECT count(url) FROM trackback WHERE entryid=%d' % eid)
        tbcount[eid] = cursor.fetchone()[0]
            
    cursor.execute(
        "SELECT to_char(date_trunc('month', date), %s), count(*), "
        "to_char(date_trunc('month', date), 'YYYYMM') "
        "FROM content GROUP BY date_trunc('month', date) ORDER BY "
        "date_trunc('month', date) DESC" % MONTHFMT)
    months = cursor.fetchall()

    header(extra_headers=extras, front=True, latest=latest)
    generate_blogtop()
    generate_sidebar(entries, topics, (not frontpage), offset, limit, count,
                     (not localref), author, topic, recent, month, months)
    generate_entries(entries, tbcount)
    footer()

def newmain():
    blog.start_timing()
    referrer = os.environ.get('HTTP_REFERER', '')
    localref = referrer and referrer.startswith(URL)
    
    cookie_str = os.environ.get('HTTP_COOKIE', '')
    try:
        cookies = Cookie.BaseCookie(cookie_str)
    except CookieError:
        cookies = Cookie.BaseCookie()

    form = ezFieldStorage()
    out, newout = sys.stdout, cStringIO.StringIO()
    sys.stdout = newout
    #print 'Content-Type: text/plain\n\n'
    #print form.items()
    resp = generate_page(localref=localref, cookies=cookies, **form)
    sys.stdout = out
    if resp:
        print 'Status: %d' % resp
    print newout.getvalue(),

# Change if using mod_python
if __name__ == '__main__':
    newmain()
