# OPML export of the blogroll
# (C) 2002-03 Chris Lawrence

import sys, cgi, psycopg, re, cStringIO, blog, db_row, blog_config
import sha
from mx import DateTime

def iso_str(date):
    return DateTime.ISO.strUTC(date).replace(' ', 'T')

def header(extra_headers=[], req=None):
    if req:
        req.content_type='text/xml; charset=utf-8'
        for (header, content) in extra_headers:
            req.headers_out.add(header, content)
    else:
        print 'Content-Type: text/xml; charset=utf-8'
        for (header, content) in extra_headers:
            print '%s: %s' % (header, content)
        print
    print '<?xml version="1.0" encoding="utf-8" ?>'

def opml(req=None):
    db = psycopg.connect(blog.DSN)
    db.commit()
    cursor = db.cursor()

    cursor.execute("SELECT name, comment, url, rssurl FROM blogroll ORDER BY lower(name)")
    R = db_row.MetaRow(cursor.description)
    blogroll = [R(row) for row in cursor.fetchall()]

    header(req=req)
    print '<opml version="1.1"><head>'
    print '<title>Blogroll for %s</title></head>' % blog_config.BLOG_TITLE
    print '<body>'
    for weblog in blogroll:
        desc = blog.recode_content(weblog.fields.comment, escape=True)
        title = blog.recode_content(weblog.fields.name, escape=True)
        rssurl = weblog.fields.rssurl
        if rssurl:
            print '<outline version="RSS" type="rss" description="%s" title="%s" xmlUrl="%s" htmlUrl="%s" />' % (desc, title, rssurl, weblog.fields.url)
##         else:
##             print '<outline type="link" description="%s" title="%s" url="%s" />' % (desc, title, weblog.fields.url)
    print '</body></opml>'
    return

def author_info(author, close=True, seealso=False):
    email = 'mailto:'+author.fields.authoremail
    sha_email = sha.new(email).hexdigest()
    
    print '<foaf:Person rdf:ID="%s">' % cgi.escape(author.fields.login)
    print ' <foaf:name>%s</foaf:name>' % cgi.escape(author.fields.authorname)
    print ' <foaf:mbox_sha1sum>%s</foaf:mbox_sha1sum>' % sha_email
    print ' <foaf:weblog rdf:resource="%s?author=%d" />' % (cgi.escape(blog.URL, 1), author.fields.author)
    if author.fields.authorurl:
        print ' <foaf:homepage rdf:resource="%s" />' % cgi.escape(author.fields.authorurl, 1)
    if author.fields.photourl:
        print ' <foaf:depiction rdf:resource="%s" />' % cgi.escape(author.fields.photourl, 1)
    icao = author.fields.icao
    iata = author.fields.iata
    if icao or iata:
        print ' <contact:nearestAirport>'
        print '  <wn:Airport'
        if icao:
            print 'air:icao="%s"' % icao
        if iata:
            print 'air:iata="%s"' % iata
        print '/>'
        print '  </contact:nearestAirport>'
    
    if seealso:
        print ' <rdf:seeAlso rdf:resource="%s?foafmode=1&amp;author=%d" />' % (
            blog.FULLOPMLURL, author.fields.author)

    if close:
        print '</foaf:Person>'

def foaf(author=None, req=None):
    db = psycopg.connect(blog.DSN)
    db.commit()
    cursor = db.cursor()

    header(req=req)
    print '''<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
  xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
  xmlns:foaf="http://xmlns.com/foaf/0.1/"
  xmlns:air="http://www.daml.org/2001/10/html/airport-ont#"
  xmlns:contact="http://www.w3.org/2000/10/swap/pim/contact#"
  xmlns:wn="http://xmlns.com/wordnet/1.6/"
  >'''

    if author:
        try:
            author = int(author)
        except:
            author = 1
        
        cursor.execute('SELECT authorname, authorurl, authoremail, icao, iata,'
                       'login, author, photourl FROM authors WHERE author=%d'
                       % author)
    else:
        cursor.execute('SELECT authorname, authorurl, authoremail, icao, iata,'
                       'login, author, photourl FROM authors ORDER BY author')
    R = db_row.MetaRow(cursor.description)
    authors = [R(row) for row in cursor.fetchall()]

    cursor.execute("SELECT name, comment, url, rssurl, foafurl FROM blogroll ORDER BY lower(name)")
    R = db_row.MetaRow(cursor.description)
    blogroll = [R(row) for row in cursor.fetchall()]

    blogrolldata = []
    for b in blogroll:
        name = blog.recode_content(b.fields.name, escape=True)
        foafurl = b.fields.foafurl
        if foafurl:
            stuff = (
                '<foaf:knows><foaf:Person>\n'
                ' <foaf:name>%s</foaf:name>\n'
                ' <rdfs:seeAlso rdf:resource="%s" />\n'
                ' <foaf:weblog rdf:resource="%s" />\n'
                '</foaf:Person></foaf:knows>'
                 % (name, cgi.escape(foafurl, 1), cgi.escape(b.fields.url, 1)))
        else:
            stuff = (
                '<foaf:knows><foaf:Person>\n'
                ' <foaf:name>%s</foaf:name>\n'
                ' <foaf:weblog rdf:resource="%s" />\n'
                '</foaf:Person></foaf:knows>'
                 % (name, cgi.escape(b.fields.url, 1)))
        blogrolldata.append(stuff)

    blogrollstuff = '\n'.join(blogrolldata)

    if len(authors) != 1:
        print '<foaf:Group>'
        print '<foaf:name>%s</foaf:name>' % cgi.escape(blog.BLOG_TITLE)
        print '<foaf:weblog>%s</foaf:weblog>' % cgi.escape(blog.URL)
        for author in authors:
            print '<foaf:member>'
            author_info(author, seealso=True)
            print '</foaf:member>'
        print blogrollstuff
        print '</foaf:Group>'
    else:
        print author_info(authors[0], close=False)
        print blogrollstuff
        print '</foaf:Person>'

    print '</rdf:RDF>'

def index(req, foafmode=None, author=None):
    out, newout = sys.stdout, cStringIO.StringIO()
    sys.stdout = newout

    #ims = req.headers_in.get('If-Modified-Since')
    try:
        if foafmode:
            foaf(author, req=req)
        else:
            opml(req=req)
        return newout.getvalue()
    finally: 
        sys.stdout = out
