#!/usr/bin/python
# -*- python -*-
#
# Python implementation of the TrackBack 1.1 spec.  Tee hee!
# (C) 2003 Chris Lawrence.
#
# Specification URL: http://www.movabletype.org/docs/mttrackback.html

import cgitb; cgitb.enable()

from blog_config import *
import cgi, os, psycopg, blog, urlparse, re, sql, pprint
import time
import urllib

# Assume trackbacks received are encoded using this charset; note that the
# spec doesn't indicate what encoding should be used...
ASSUME_ENCODING='utf-8'

LEGACY_MODE=True

DEBUG=True

def xml_header():
    print 'Content-Type: text/xml'
    print
    print '<?xml version="1.0" encoding="utf-8"?>'
    print '<response>'

def xml_footer():
    print '</response>'

def error_message(message):
    print 'Status: 400'
    xml_header()
    print '<error>1</error>'
    print '<message>%s</message>' % message
    xml_footer()

def connect():
    try:
        db = psycopg.connect(DSN)
    except:
        error_message('Database is currently down; please try again in a few '
                      'minutes.')
        sys.exit(0)
    return db

#striphtmlre = re.compile(r'</?.*?>', re.DOTALL)
#quotesre = re.compile(r'&[lr]dquo;', re.DOTALL)

def list_tb_pings(tid, html=False):
    db = connect()
    cursor = db.cursor()
    cursor.execute('SELECT title, body FROM content WHERE entryid=%d' % tid)
    try:
        entry_title, desc = cursor.fetchone()[:2]
    except:
        error_message('Entry ID appears to be invalid.')
        return
    cursor.execute('SELECT title, url, excerpt, blog_name, to_char(added, %s) '
                   'FROM trackback WHERE entryid=%d ORDER BY added' % (
        DATEFMT, tid))
    pings = cursor.fetchall()
    db.close()

    if html:
        tburl = '%s/%d' % (TBURL, tid)
        print 'Content-Type: text/html; charset=utf-8'
        print
        print '<?xml version="1.0" encoding="utf-8"?>'
        print '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
        print '<html xmlns="http://www.w3.org/1999/xhtml"><head>'
        print '<link rel="stylesheet" type="text/css" href="%s" />' % \
              STYLESHEETS[DEF_STYLESHEET][1]
        print '<meta name="robots" content="noindex,follow" />'
        print '<title>%s: Discussion on %s</title>' % (cgi.escape(BLOG_TITLE),
                                                       cgi.escape(entry_title))
        print '</head>'
        print '<body>'
        print '<p>Manual <a href="http://www.movabletype.org/trackback/"'
        print '>TrackBack</a> URL for this entry: %s</p>' % tburl
        #print '<p>An <a href="%s/%d?__mode=rss">RSS feed</a> is also available for this entry.</p>' % (TBURL, tid)
        print '<p>Listed below are weblogs that have linked to <a href="%s?entryid=%d">%s</a>'% (URL, tid, cgi.escape(entry_title))
        print 'at <a href="%s">%s</a>:</p>' % (URL, cgi.escape(BLOG_TITLE))
        print '<ul>'
        if not pings:
            print '<li>No trackbacks found.</li>'
        for ping in pings:
            title, url, excerpt, blog_name, added = ping[:5]
            print '<li><a href="%s">%s</a><br />' % (url, cgi.escape(title))
            print '<strong>Excerpt:</strong> %s<br />' % cgi.escape(excerpt)
            print '<strong>Weblog:</strong> %s<br />' % cgi.escape(blog_name)
            print '<strong>Tracked:</strong> %s</li>' % cgi.escape(added)
        print '</ul>'
        print '<p><a href="http://www.aylwardfamily.com/content/tbping.asp?TBURL=%s">Register a trackback manually</a>.</p>' % urllib.quote_plus(tburl)
        print '''\
<p>(Please note that this is a non-Movable Type implementation of the
specification.  Contact lsblog -at- lordsutch.com if your ping fails
to work correctly.)</p>
</body></html>'''
    else:
        desc = blog.make_excerpt(desc)
        xml_header()
        print '<error>0</error>'
        print '<rss version="0.91"><channel>'
        print '<title>%s</title>' % cgi.escape(entry_title)
        print '<link>%s?entryid=%d</link>' % (URL, tid)
        print '<description>%s</description>' % desc
        print '<language>%s</language>' % LANG
        for ping in pings:
            title, url, excerpt, added = ping[:4]
            print '<item>'
            print '<title>%s</title><link>%s</link>' % (title, url)
            print '<description>%s</description>' % excerpt
            print '</item>'
        print '</channel></rss>'
        xml_footer()

def receive_tb_ping(tid, form):
    url = form.getfirst('url', '')
    if not url:
        list_tb_pings(tid, True)
        return

    ip = os.environ.get('REMOTE_ADDR', '0.0.0.0')

    if DEBUG:
        fp = open('/tmp/tb.log', 'a')
        print >> fp, 'Receiving trackback from %s (%s)' % (
            ip, os.environ.get('REMOTE_HOST', 'unknown'))
##         for k in form.keys():
##             pprint.pprint((k, form.getfirst(k, '')), stream=fp)
##         print >> fp, 'here'
    
    # Try to quote out anything mischevious
    bits = urlparse.urlparse(url)
    if bits[0] != 'http' and bits[0] != 'https':
        error_message('Only HTTP URLs are accepted.')
        return
    # Clean up any weirdness in the URL
    url = urlparse.urlunparse(bits)[:128]
    
    title = form.getfirst('title', '')
    excerpt = form.getfirst('excerpt', '')
    blog_name = form.getfirst('blog_name', '')

    if not title:
        title = '(no title)'
    if not excerpt:
        excerpt = '(none)'
    if not blog_name:
        blog_name = '(Unknown weblog)'

    # Treat text as ASSUME_ENCODING, replacing any garbage if needed
    title = title.decode(ASSUME_ENCODING, 'replace')
    excerpt = excerpt.decode(ASSUME_ENCODING, 'replace')
    blog_name = blog_name.decode(ASSUME_ENCODING, 'replace')

    # Strip any HTML
    title = blog.strip_html(title)[:128]
    excerpt = blog.strip_html(excerpt)
    blog_name = blog.strip_html(blog_name)[:128]

    # Truncate excerpt    
    if len(excerpt) > 255:
        excerpt=excerpt[:253]+'...'

    query = sql.Insert("trackback", {'entryid' : tid, 'title' : title,
                                     'excerpt' : excerpt, 'url' : url,
                                     'blog_name' : blog_name, 'ip' : ip})

    if DEBUG:
        print >> fp, 'Trackback to:', tid, time.ctime()
        pprint.pprint(str(query), stream=fp)

    db = connect()
    db.commit()
    cursor = db.cursor()
    eurl = url.replace("'", r"\'")
    cursor.execute("SELECT * FROM trackback WHERE entryid=%d AND "
                   "url='%s'" % (tid, eurl))
    entries = cursor.fetchall()
    if not entries:
        if DEBUG:
            print >> fp, 'Trackback not a duplicate.'
        try:
            cursor.execute(str(query))
            db.commit()
            xml_header()
            print '<error>0</error>'
            xml_footer()
            if DEBUG:
                print >> fp, 'Trackback OK.\n'
        except:
            xml_header()
            print '<error>1</error>'
            print '<message>SQL insert failed.  Check debug log.</message>'
            xml_footer()
            if DEBUG:
                traceback.print_exc(file=fp)
    else:
        error_message('Duplicate trackback received.')
        if DEBUG:
            print >> fp, 'Duplicate trackback.\n'

    db.close()
    fp.close()

def main():
    reqmeth = os.environ.get('REQUEST_METHOD', 'GET')
    path = os.environ.get('PATH_INFO')
    form = cgi.FieldStorage()
    tid = None
    if path:
        try:
            tid = int(path.split('/', 2)[1])
        except:
            tid = None

    mode = form.getfirst('__mode', '')

    if not tid:
        print 'Status: 400\nContent-Type: text/plain'
        print
        print 'Bad request - a TrackBack ID is required in the URI'
        #print path
    elif reqmeth == 'GET':
        if not LEGACY_MODE or (LEGACY_MODE and mode=='rss'):
            list_tb_pings(tid)
        elif mode == 'view':
            list_tb_pings(tid, html=True)
        elif LEGACY_MODE:
            receive_tb_ping(tid, form)
    elif reqmeth == 'POST':
        receive_tb_ping(tid, form)
    else:
        print 'Status: 400\nContent-Type: text/plain'
        print
        print 'Bad request - only GET and POST supported'
    return

if __name__ == '__main__':
    main()
